'use strict';

Object.defineProperty(exports, "__esModule", {
	value: true
});

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

var _bcryptjs = require('bcryptjs');

var _bcryptjs2 = _interopRequireDefault(_bcryptjs);

var _jsonwebtoken = require('jsonwebtoken');

var _jsonwebtoken2 = _interopRequireDefault(_jsonwebtoken);

var _ = require('..');

var _2 = _interopRequireDefault(_);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

var SALT_ROUNDS = _2.default.config.get('auth.saltRounds');
var SECRET = _2.default.config.get('auth.secret');

/**
 * Auth Class
 */

var Auth = function () {
	function Auth() {
		_classCallCheck(this, Auth);
	}

	_createClass(Auth, [{
		key: 'hash',


		/**
   * Hash a given string with salt
   * 
   * @param  {String} str the unencrytped string
   * @return {Promise}
   */
		value: function hash(str) {
			return new Promise(function (resolve, reject) {
				_bcryptjs2.default.hash(str, SALT_ROUNDS, function (err, hash) {
					if (err) {
						return reject(err);
					}

					return resolve(hash);
				});
			});
		}

		/**
   * Compare a string and a hash to check equality
   * 
   * @param  {String} str  the unencrypted string
   * @param  {String} hash the hash to compare with the string
   * @return {Promise}
   */

	}, {
		key: 'compare',
		value: function compare(str, hash) {
			return new Promise(function (resolve, reject) {
				_bcryptjs2.default.compare(str, hash, function (err, res) {
					if (err) {
						return reject(err);
					}

					return resolve(!!res);
				});
			});
		}

		/**
   * Sign JWT
   * 
   * @param  {Object} obj 
   * @return {String}     token
   */

	}, {
		key: 'sign',
		value: function sign(obj) {
			return _jsonwebtoken2.default.sign(obj, SECRET);
		}

		/**
   * Verify JSON Web Token
   * 
   * @param  {String} token jwt
   * @return {Mixed}
   */

	}, {
		key: 'verify',
		value: function verify(token) {
			try {
				var decoded = _jsonwebtoken2.default.verify(token, SECRET);
				return decoded;
			} catch (err) {
				// TODO use logging library instead
				console.error(err);
				return false;
			}
		}
	}]);

	return Auth;
}();

exports.default = new Auth();