'use strict';

Object.defineProperty(exports, "__esModule", {
	value: true
});

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

var _moment = require('moment');

var _moment2 = _interopRequireDefault(_moment);

var _ = require('..');

var _2 = _interopRequireDefault(_);

var _Query = require('./Query');

var _Query2 = _interopRequireDefault(_Query);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

var Model = function () {

	/**
  * Constructor
  * 
  * @constructor
  * @param  {Object} props Initial properties
  * @return {Model}
  */


	/**
  * Fillable Attributes
  * @type {Array}
  */

	function Model(props) {
		_classCallCheck(this, Model);

		this.fill(props);
	}

	/**
  *************************************************
  * Utilities
  *************************************************
  */

	/**
  * Mass Assignment
  *
  * will only assign properties which are whitelisted
  * in the static fillable property
  * 
  * @param  {Object} props properties to assign
  * @return {Model}
  */


	/**
  * Primary Key Name
  * @type {String}
  */


	/**
  *************************************************
  * Configuration
  *************************************************
  */

	/**
  * Table Name
  * @type {String}
  */


	_createClass(Model, [{
		key: 'fill',
		value: function fill(props) {
			for (var key in props) {
				if (this.isFillable(key)) {
					this.set(key, props[key]);
				}
			}

			return this;
		}

		/**
   * Check if a key is fillable
   * 
   * @param  {String}  key name of the property in question
   * @return {Boolean}
   */

	}, {
		key: 'isFillable',
		value: function isFillable(key) {
			var fillable = this.getFillable();

			return fillable.indexOf(key) >= 0;
		}

		/**
   * Get Fillable Attributes
   * 
   * @return {Array} array of fillable attributes
   */

	}, {
		key: 'getFillable',
		value: function getFillable() {
			return this.constructor.fillable;
		}

		/**
   * Get the name of primary key
   * 
   * @return {String} name of primary key
   */

	}, {
		key: 'primaryKey',
		value: function primaryKey() {
			return this.constructor.primaryKey;
		}

		/**
   * Get the value of the primary key
   * 
   * @return {Mixed} the primary key value
   */

	}, {
		key: 'getPrimaryKey',
		value: function getPrimaryKey() {
			return this.get(this.primaryKey());
		}

		/**
   * Get Model Attribute
   * 
   * @param  {String} key name of the value
   * @return {Mixed}
   */

	}, {
		key: 'get',
		value: function get(key) {
			return this[key];
		}

		/**
   * Set Model Attribute
   * 
   * @param {String} key name of the value
   * @param {Model}
   */

	}, {
		key: 'set',
		value: function set(key, val) {
			this[key] = val;

			return this;
		}

		/**
   * Convert this model to JSON
   * 
   * @return {Object} JSON-friendly object
   */

	}, {
		key: 'toJSON',
		value: function toJSON() {
			var json = {};

			for (var key in this) {
				if (this.isFillable(key)) {
					var val = this.get(key);

					if (val instanceof _moment2.default) {
						val = val.format('YYYY/MM/DD HH:mm:ss');
					}

					json[key] = val;
				}
			}

			return json;
		}

		/**
   * Return a timestamp
   *
   * @default Date.time()
   * @param  {Date} date
   * @return {Moment}
   */

	}, {
		key: 'timestamp',
		value: function timestamp() {
			var date = arguments.length <= 0 || arguments[0] === undefined ? Date.now() : arguments[0];

			return (0, _moment2.default)(date);
		}

		/**
   * Get the table for this model
   * 
   * @return {String}
   */

	}, {
		key: 'getTable',
		value: function getTable() {
			return this.constructor.getTable();
		}

		/**
   *************************************************
   * DB Related Methods
   *************************************************
   */
		// TODO refactor this to use the query builder

	}, {
		key: 'save',
		value: function save() {
			var _this = this;

			var query = _2.default.db(this.getTable());

			if (this.get('id')) {
				this.set('updated_at', this.timestamp());

				query.where('id', id).update(this.toJSON());
			} else {
				this.set('created_at', this.timestamp());
				this.set('updated_at', this.timestamp());

				query.insert(this.toJSON());
			}

			return new Promise(function (resolve, reject) {
				query.then(function (ids) {
					_this.set('id', ids[0]);
					return resolve(_this);
				});
			});
		}

		/**
   *************************************************
   * Relationships
   *************************************************
   */
		// TODO refactor this to use the query builder

	}, {
		key: 'hasMany',
		value: function hasMany(Model, fk) {
			var _this2 = this;

			return new Promise(function (resolve, reject) {
				_2.default.db(Model.getTable()).where(fk, _this2.id).then(function (results) {
					resolve(results.map(function (result) {
						return new Model(result);
					}));
				}).catch(reject);
			});
		}

		// TODO refactor this to use the query builder

	}, {
		key: 'belongsToMany',
		value: function belongsToMany(Related, pivot, fk, ofk) {
			var _this3 = this;

			var table = Related.getTable();

			return new Promise(function (resolve, reject) {
				_2.default.db(pivot).select(table + '.*').where(fk, _this3.id).leftJoin(table, pivot + '.' + ofk, '=', table + '.id').then(function (results) {
					resolve(results.map(function (result) {
						return new Related(result);
					}));
				}).catch(reject);
			});
		}

		// TODO refactor this to use the query builder

	}, {
		key: 'hasOne',
		value: function hasOne() {
			return this.belongsTo.apply(this, arguments);
		}

		// TODO refactor this to use the query builder

	}, {
		key: 'belongsTo',
		value: function belongsTo(Related, fk) {
			return Related.where('id', this[fk]).first();
		}

		/**
   *************************************************
   * Static Utilities
   *************************************************
   */

	}], [{
		key: 'instance',
		value: function instance() {
			for (var _len = arguments.length, args = Array(_len), _key = 0; _key < _len; _key++) {
				args[_key] = arguments[_key];
			}

			return new (Function.prototype.bind.apply(this, [null].concat(args)))();
		}
	}, {
		key: 'getTable',
		value: function getTable() {
			return this.table;
		}
	}, {
		key: 'getPrimaryKey',
		value: function getPrimaryKey() {
			return this.instance().primaryKey;
		}

		/**
   *************************************************
   * Static DB Related Methods
   *************************************************
   */

	}, {
		key: 'find',
		value: function find(id) {
			return new _Query2.default(this).table(this.getTable()).where('id', id).first();
		}
	}, {
		key: 'all',
		value: function all() {
			return new _Query2.default(this).table(this.getTable()).orderBy('created_at', 'desc').get();
		}
	}, {
		key: 'paginate',
		value: function paginate() {
			return new _Query2.default(this).table(this.getTable()).orderBy('created_at', 'desc').get();
		}
	}, {
		key: 'create',
		value: function create(data) {
			return this.instance(data).save();
		}
	}, {
		key: 'destroy',
		value: function destroy(ids) {
			return new Promise(function (resolve, reject) {
				resolve(true);
			});
		}
	}, {
		key: 'where',
		value: function where() {
			var _table;

			return (_table = new _Query2.default(this).table(this.getTable())).where.apply(_table, arguments);
		}
	}]);

	return Model;
}();

Model.table = '';
Model.fillable = [];
Model.primaryKey = 'id';
exports.default = Model;