'use strict';

Object.defineProperty(exports, "__esModule", {
	value: true
});
exports.bumpVersion = bumpVersion;
exports.changelog = changelog;
exports.commitChanges = commitChanges;
exports.pushChanges = pushChanges;
exports.createNewTag = createNewTag;
exports.githubRelease = githubRelease;
exports.default = release;
function getPackageJsonVersion() {
	return JSON.parse(fs.readFileSync('./package.json', 'utf8')).version;
}

function bumpVersion() {
	return gulp.src(['./bower.json', './package.json']).pipe(bump({ type: "patch" }).on('error', gutil.log)).pipe(gulp.dest('./'));
}

function changelog() {
	return gulp.src('CHANGELOG.md', {
		buffer: false
	}).pipe(conventionalChangelog({
		preset: 'angular'
	})).pipe(gulp.dest('./'));
}

function commitChanges() {
	return gulp.src('.').pipe(git.add()).pipe(git.commit('[Prerelease] Bumped version number'));
}

function pushChanges(cb) {
	git.push('origin', 'master', cb);
}

function createNewTag() {
	var version = getPackageJsonVersion();

	git.tag(version, 'Created Tag for version: ' + version, function (error) {
		if (error) {
			return cb(error);
		}
		git.push('origin', 'master', { args: '--tags' }, cb);
	});
}

function githubRelease(cb) {
	conventionalGithubReleaser({
		type: "oauth",
		token: '0126af95c0e2d9b0a7c78738c4c00a860b04acc8'
	}, {
		preset: 'angular'
	}, cb);
}

function release(cb) {
	runSequence('bumpVersion', 'changelog', 'commitChanges', 'pushChanges', 'createNewTag', 'githubRelease', function (err) {
		if (err) {
			console.log(err.message);
		} else {
			console.log('RELEASE FINISHED SUCCESSFULLY');
		}
		cb(err);
	});
}