'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.slug = slug;
exports.truncate = truncate;
exports.stripMarkdown = stripMarkdown;
exports.snippet = snippet;

var _removeMarkdown = require('remove-markdown');

var _removeMarkdown2 = _interopRequireDefault(_removeMarkdown);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

/**
 * Create a slug from a given string
 * 
 * @param  {String} str original string
 * @return {String}     slug
 */
function slug() {
  var str = arguments.length <= 0 || arguments[0] === undefined ? '' : arguments[0];

  return str.toLowerCase().replace(/ /ig, '-');
}

/**
 * Truncate a given string
 * 
 * @param  {String} str original string
 * @param  {Int} len new length of string
 * @return {String}
 */
function truncate() {
  var str = arguments.length <= 0 || arguments[0] === undefined ? '' : arguments[0];
  var len = arguments.length <= 1 || arguments[1] === undefined ? 250 : arguments[1];

  return str.substr(0, len);
}

/**
 * Remove markdown from string
 * 
 * useful for showing previews of markdown items
 * 
 * @param  {String} str original string with markdown
 * @return {String}     string clean of any markdown
 */
function stripMarkdown() {
  var str = arguments.length <= 0 || arguments[0] === undefined ? '' : arguments[0];

  return (0, _removeMarkdown2.default)(str);
}

/**
 * Create a "snippet"
 * 
 * will remove any markdown from the string, then truncate the
 * remaining text to the desired length
 * 
 * @param  {String} str original string
 * @param  {Int} len new length of string
 * @return {String}     
 */
function snippet() {
  var str = arguments.length <= 0 || arguments[0] === undefined ? '' : arguments[0];
  var len = arguments[1];

  return truncate(stripMarkdown(str), len);
}